#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <Ecore.h>

#include "Emusic.h"

#define PLAYLIST_NAME "test_playlist"
#define CONTROLLER_NAME "test_controller"

static Ecore_List *_emusic_playlist_populate(char *dir);

int main(int argc, char** argv)
{
  int i = 0;
  char *modulename = calloc(1, 128);
  char *path       = calloc(1, 128);
  char *fd, *pfile = NULL;

  Ecore_List *ls = NULL;
  Emusic_Playlist *playlist = NULL;
  Emusic_Controller *controller = NULL;


  printf("EMUSIC TEST STARTING :\n");
  for ( i = 0; i < argc; i++ )
    {
      if ( !strcasecmp(argv[i], "-m") || !strcasecmp(argv[i], "--module") )
        {
          if ( argv[i+1] )
            {
              sprintf(modulename, "emusic_%s_plugin", argv[i+1]);
              printf(" - using selected module : %s\n", modulename);
            }
          else
            {
              sprintf(modulename, "emusic_gstreamer_plugin");
              printf(" - using default module : %s\n", modulename);
            }
        }
      if ( !strcasecmp(argv[i], "-d") || !strcasecmp(argv[i], "--directory") )
        {
          if ( argv[i+1] )
            {
              sprintf(path, "%s", argv[i+1]);
              if ( ! ecore_file_exists( path ) )
                {
                  free(path);
                  sprintf(path, "./");
                  printf(" - using default directory : %s\n", path);
                }
              else
                {
                  printf(" - using selected directory : %s\n", path);
                }
            }
        }
    }

  if (!emusic_init(modulename))
    {
      printf("ERROR: Init Emusic\n");
      return -1;
    }

  controller = emusic_controller_new(CONTROLLER_NAME);

  if ( ! emusic_controller_mode_set(controller, EMUSIC_CONTROLLER_MODE_PLAYER) )
    {
      printf("ERROR: Can't set the player mode to controller %s.\n", controller->name);
      return -1;
    }

  if ( ! emusic_controller_playlist_new(controller, PLAYLIST_NAME) )
    {
      printf("ERROR: Can't create a new playlist for %s.\n", controller->name);
    }

  printf("\n====================================================\n");
  printf("Adding File To Playlist:\n");
  ls = _emusic_playlist_populate(path);

  if ( ! ls )
    return -1;

  while ( ( fd = ecore_list_next(ls) ) )
    {
      pfile = (char *)calloc(1, (strlen(path)+strlen(fd)+2));
      sprintf(pfile, "%s/%s", path, fd);
      printf("filename :%s\n", pfile);
      if ( ! emusic_controller_playlist_add(controller, pfile, PLAYLIST_NAME) )
        {
          printf("Warning : couldn't add %s to playlist\n", pfile);
          continue;
        }
      else
        {
          printf("File %s added to playlist %s.\n", pfile, PLAYLIST_NAME);
        }
    }
  printf("\n====================================================\n");
  printf("Getting Files From Playlist:\n");
  playlist = emusic_controller_playlist_get(controller, PLAYLIST_NAME);
  if ( ! playlist )
    {
      printf("ERROR: getting playlist found a NULL ptr.\n");
      return -1;
    }
  printf("Found 1 : %s\n", playlist->name);

  ecore_list_goto_first(playlist->files);
  while ( ( fd = ecore_list_next(playlist->files) ) )
    printf("File found %s on playlist %s.\n", fd, playlist->name);

  printf("\n====================================================\n");


  printf("Starting playing...\n");
  if ( ! emusic_controller_play(controller) )
    {
      printf("ERROR : could not play.\n");
    }

  printf("Volume atm : %lf\n", emusic_controller_volume_get(controller));

  ecore_main_loop_begin();

  free(modulename);
  free(path);
  free(fd);
  if (pfile)
    free(pfile);

  return 0;
}


static Ecore_List *_emusic_playlist_populate(char *dir)
{
  Ecore_List *playlist = NULL;
  Ecore_List *ls = NULL;
  char *file;

  if ( ! ecore_file_exists(dir) )
    return NULL;

  ls = ecore_file_ls(dir);
  playlist = ecore_list_new();
  ecore_list_init(playlist);

  if ( ! ls )
    return NULL;

  ecore_list_goto_first(ls);
  while ( ( file = ecore_list_next(ls) ) )
    {
      if ( strstr(file, ".mp3") )
        {
          ecore_list_append(playlist, file);
        }
    }

  ecore_list_goto_first(playlist);
  return playlist;
}
