#ifndef EMUSIC_PRIVATE_H
#define EMUSIC_PRIVATE_H

#include <Ecore.h>

#include "config.h"
#include "Emusic.h"

#define NBR_ELEMENTS(array) (sizeof (array) / sizeof ((array)[0]))



typedef struct _Emusic_Module Emusic_Module;
typedef struct _Emusic Emusic;


enum _EMUSIC_SIGNALS
{
  EMUSIC_SIGNAL_EOS,
  EMUSIC_SIGNAL_EOPL,
  EMUSIC_SIGNAL_NEW_TAGS,
  EMUSIC_SIGNAL_NEW_FILE,
  EMUSIC_SIGNAL_NBR
};

struct _Emusic_Module
  {
    int                    (*init)                           (void **module);
    int                    (*shutdown)                       (void *module);

    Emusic_Controller     *(*controller_new)                 (void *module, char *name);
    int                    (*controller_remove)              (void *module, Emusic_Controller *controller);
    char                  *(*controller_name_get)            (void *module, Emusic_Controller *controller);
    int                    (*controller_name_set)            (void *module, Emusic_Controller *controller, char *name);
    Emusic_Controller_Mode (*controller_mode_get)            (void *module, Emusic_Controller *controller);

    int                    (*controller_mode_set)            (void *module, Emusic_Controller *controller, Emusic_Controller_Mode mode);
    int                    (*controller_playlist_new)        (void *module, Emusic_Controller *controller, char *playlist_name);
    int                    (*controller_playlist_add)        (void *module, Emusic_Controller *controller, char *filename, char *playlist_name);
    int                    (*controller_playlist_remove)     (void *module, Emusic_Controller *controller, char *filename);
    int                    (*controller_playlist_remove_id)  (void *module, Emusic_Controller *controller, int id);
    int                    (*controller_playlist_goto)       (void *module, Emusic_Controller *controller, char *filename);
    int                    (*controller_playlist_goto_id)    (void *module, Emusic_Controller *controller, int id);
    int                    (*controller_playlist_destroy)    (void *module, Emusic_Controller *controller, char *playlist_name);
    int                    (*controller_playlist_load)       (void *module, Emusic_Controller *controller, char *filename);
    int                    (*controller_playlist_save)       (void *module, Emusic_Controller *controller, char *filename);
    Emusic_Playlist       *(*controller_playlist_get)        (void *module, Emusic_Controller *controller, char *playlist_name);

    int                    (*controller_play)                (void *module, Emusic_Controller* controller);
    int                    (*controller_pause)               (void *module, Emusic_Controller* controller);
    int                    (*controller_stop)                (void *module, Emusic_Controller* controller);
    int                    (*controller_next)                (void *module, Emusic_Controller* controller);
    int                    (*controller_prev)                (void *module, Emusic_Controller* controller);

    double                 (*controller_volume_get)          (void *module, Emusic_Controller* controller);
    int                    (*controller_volume_set)          (void *module, Emusic_Controller* controller, double volume);
    int                    (*controller_mute_get)            (void *module, Emusic_Controller* controller);
    int                    (*controller_mute_set)            (void *module, Emusic_Controller* controller, int mute);

    double                 (*controller_length_get)          (void *module, Emusic_Controller* controller);
    double                 (*controller_position_get)        (void *module, Emusic_Controller* controller);
    int                    (*controller_position_set)        (void *module, Emusic_Controller* controller, double position);
  };

struct _Emusic
  {
    Ecore_Plugin  *plugin;
    Emusic_Module *module;
    void          *priv;
    int            group_id;
  };

#endif


