#include "xmms2_plugin.h"


Emusic_Module mod =
  {
    mod_init,
    mod_shutdown,
    mod_controller_new,
    mod_controller_remove,
    mod_controller_name_get,
    mod_controller_name_set,
    mod_controller_mode_get,
    mod_controller_mode_set,
    mod_controller_playlist_new,
    mod_controller_playlist_add,
    mod_controller_playlist_remove,
    mod_controller_playlist_destroy,
    mod_controller_playlist_save,
    mod_controller_playlist_get,
    mod_controller_play,
    mod_controller_pause,
    mod_controller_stop,
    mod_controller_next,
    mod_controller_prev,
    mod_controller_volume_get,
    mod_controller_volume_set,
    mod_controller_length_get
  };

int
mod_init(void **data)
{
  Emusic_Xmms2_Private *priv = NULL;

  priv = calloc(1, sizeof(Emusic_Xmms2_Private));
  if ( ! priv )
    {
      printf("Error, failed allocating Xmms2\n");
      return 0;
    }

  *data = priv;

  priv->controllers = ecore_list_new();
  ecore_list_set_free_cb(priv->controllers, _xmms_controller_remove_destroy_cb);
  ecore_list_init(priv->controllers);
  priv->controllers_count = 0;


  priv->con = xmmsc_init(EMUSIC_XMMS2_CLIENT_NAME);

  priv->ipc_path = getenv ("XMMS_PATH");

  if ( ! xmmsc_connect(priv->con, priv->ipc_path) )
    {
      printf("XMMS2..: A problem append initializing the xmms2 connection to daemon...\n"
             "         Maybe you do not start it.\n"
             "Error..: %s\n", xmmsc_get_last_error(priv->con));
      return 0;
    }

  return 1;
}

int
mod_shutdown(void *data)
{
  if ( ! data )
    return 0;

  Emusic_Xmms2_Private *priv;
  priv = (Emusic_Xmms2_Private *)data;
  /*
  	priv->res = xmmsc_quit(priv->con);
  	if (xmmsc_result_iserror(priv->res))
  	{
  		_xmms_print_iserror(priv->res, __func__, __LINE__);
  		return 0;
  	}
   
  	xmmsc_unref(priv->con);
   
  	EMUSIC_FREE(priv);
  */
  return 1;
}

Emusic_Controller*
mod_controller_new(void *data, char* name)
{
  if ( ! data )
    return NULL;

  Emusic_Xmms2_Private *priv;
  Emusic_Controller *controller;

  priv = (Emusic_Xmms2_Private *)data;
  controller = (Emusic_Controller *)calloc(1, sizeof(Emusic_Controller));
  if (!controller)
    return NULL;


  if (!name)
    {
      char buf[256];

      snprintf(buf, sizeof(buf), "controller-%d", priv->controllers_count);
      controller->name = strdup(buf);
    }
  else
    {
      controller->name = strdup(name);
    }

  controller->mode = EMUSIC_CONTROLLER_MODE_NULL;
  controller->playlists = ecore_list_new();
  ecore_list_init(controller->playlists);
  ecore_list_append(priv->controllers, controller);
  priv->controllers_count++;

  printf("Nomber of controllers : %d\n", priv->controllers_count);

  return controller;
}

char*
mod_controller_name_get(void *data, Emusic_Controller *controller)
{
  if ( !data || !controller )
    return NULL;

  return controller->name;
}

int
mod_controller_name_set(void *data, Emusic_Controller *controller, char* name)
{
  if ( !data || !controller || !name )
    return 0;

  free(controller->name);
  controller->name = (char *)calloc(1, strlen(name));
  sprintf(controller->name, "%s", name);

  if ( strcasecmp(controller->name, name) )
    {
      printf("Internal error, can't change the controller name.\n");
      return 0;
    }

  return 1;
}

Emusic_Controller_Mode
mod_controller_mode_get(void *data, Emusic_Controller *controller)
{
  if ( !data || !controller )
    return EMUSIC_CONTROLLER_MODE_NULL;

  return controller->mode;
}

int
mod_controller_mode_set(void *data, Emusic_Controller *controller, Emusic_Controller_Mode mode)
{
  if ( !data || !controller )
    return 0;

  controller->mode = mode;

  return 1;
}

int
mod_controller_remove(void *data, Emusic_Controller *controller)
{
  if ( ! data || ! controller )
    return 0;

  Emusic_Xmms2_Private *priv;
  Emusic_Controller *ec = NULL;

  priv = (Emusic_Xmms2_Private *)data;

  ec = ecore_list_goto(priv->controllers, controller);
  if (ec != controller)
    return 0;

  priv->controllers_count--;

  return ecore_list_remove_destroy(priv->controllers);
}

int
mod_controller_play(void *data, Emusic_Controller *controller)
{
  if ( ! data || !controller )
    return 0;

  Emusic_Xmms2_Private *priv;

  priv = (Emusic_Xmms2_Private *)data;


  priv->res = xmmsc_playback_start(priv->con);
  xmmsc_result_wait(priv->res);

  if (xmmsc_result_iserror(priv->res))
    {
      _xmms_print_iserror(priv->res, __func__, __LINE__);
      return 0;
    }
  xmmsc_result_unref(priv->res);

  return 1;
}

int
mod_controller_pause(void *data, Emusic_Controller *controller)
{
  if ( ! data || !controller )
    return 0;

  Emusic_Xmms2_Private *priv;

  priv = (Emusic_Xmms2_Private *)data;


  priv->res = xmmsc_playback_pause(priv->con);
  xmmsc_result_wait(priv->res);

  if (xmmsc_result_iserror(priv->res))
    {
      _xmms_print_iserror(priv->res, __func__, __LINE__);
      return 0;
    }
  xmmsc_result_unref(priv->res);

  return 1;
}

int
mod_controller_stop(void *data, Emusic_Controller *controller)
{
  if ( ! data || !controller )
    return 0;

  Emusic_Xmms2_Private *priv;

  priv = (Emusic_Xmms2_Private *)data;


  priv->res = xmmsc_playback_stop(priv->con);
  xmmsc_result_wait(priv->res);

  if (xmmsc_result_iserror(priv->res))
    {
      _xmms_print_iserror(priv->res, __func__, __LINE__);
      return 0;
    }
  xmmsc_result_unref(priv->res);

  return 1;
}

int
mod_controller_next(void *data, Emusic_Controller *controller)
{
  if ( ! data || !controller )
    return 0;

  Emusic_Xmms2_Private *priv;
  priv = (Emusic_Xmms2_Private *)data;

  return _xmms_controller_jump_to(priv, 1);
}


int
mod_controller_prev(void *data, Emusic_Controller *controller)
{
  if ( ! data || !controller )
    return 0;

  Emusic_Xmms2_Private *priv;
  priv = (Emusic_Xmms2_Private *)data;

  return _xmms_controller_jump_to(priv, -1);
}

double
mod_controller_volume_get(void *data, Emusic_Controller *controller)
{
  if ( ! data || !controller )
    return 0;

  int l_vol = 0;
  Emusic_Xmms2_Private *priv;
  priv = (Emusic_Xmms2_Private *)data;


  priv->res = xmmsc_playback_volume_get(priv->con);
  xmmsc_result_wait(priv->res);

  if (xmmsc_result_iserror(priv->res))
    {
      _xmms_print_iserror(priv->res, __func__, __LINE__);
      return 0;
    }

  if (!xmmsc_result_get_dict_entry_uint32(priv->res, "left", &l_vol))
    {
      printf("Can't get current volume...\n");
      return 0;
    }

  xmmsc_result_unref(priv->res);

  return (double )l_vol;
}

int
mod_controller_volume_set(void *data, Emusic_Controller *controller, double vol)
{
  if ( ! data || !controller )
    return 0;

  Emusic_Xmms2_Private *priv;
  priv = (Emusic_Xmms2_Private *)data;

  priv->res = xmmsc_playback_volume_set (priv->con, "left", (int)vol);
  xmmsc_result_wait(priv->res);

  if (xmmsc_result_iserror(priv->res))
    {
      _xmms_print_iserror(priv->res, __func__, __LINE__);
      return 0;
    }
  xmmsc_result_unref(priv->res);

  priv->res = xmmsc_playback_volume_set (priv->con, "right", (int)vol);
  xmmsc_result_wait(priv->res);

  if (xmmsc_result_iserror(priv->res))
    {
      _xmms_print_iserror(priv->res, __func__, __LINE__);
      return 0;
    }
  xmmsc_result_unref(priv->res);

  return 1;
}

double
mod_controller_length_get(void *data, Emusic_Controller *controller)
{
  return 0.0;
}





/**
 * MODULE API ROUTINES
 */

unsigned char
module_open(Emusic_Module **data, void** priv)
{
  if (!data || !priv)
    {
      printf("Module Adress == NULL\n");
      return 0;
    }

  if (!mod.init(priv))
    {
      printf("mod.init return 0\n");
      return 0;
    }

  *data = &mod;

  return 1;
}


void
module_close(Emusic_Module *data)
{
  Emusic_Xmms2_Private *priv;

  priv = (Emusic_Xmms2_Private *)data;

  mod.shutdown(priv);
}





/**
 * INTERNAL API
 */


int _xmms_controller_jump_to(Emusic_Xmms2_Private *priv, int pos)
{
  if ( ! priv || !pos )
    return 0;

  priv->res = xmmsc_playlist_set_next_rel(priv->con, pos);
  xmmsc_result_wait(priv->res);

  if (xmmsc_result_iserror (priv->res))
    {
      _xmms_print_iserror(priv->res, __func__, __LINE__);
      return 0;
    }
  xmmsc_result_unref (priv->res);

  priv->res = xmmsc_playback_tickle(priv->con);
  xmmsc_result_wait(priv->res);

  if (xmmsc_result_iserror (priv->res))
    {
      _xmms_print_iserror(priv->res, __func__, __LINE__);
      return 0;
    }
  xmmsc_result_unref (priv->res);

  return 1;
}
